package controller

import (
	"cold-logistics/app/admin/model"
	"cold-logistics/app/admin/service"
	"cold-logistics/app/admin/service/dto"
	"cold-logistics/common/actions"
	"errors"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type AddressController struct {
	api.Api
}

// GetPage 获取地址列表
// @Summary 获取地址列表
// @Description 获取地址列表
// @Tags 地址
// @Param pageSize query int false "页条数"
// @Param page query int false "页码"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.Address}} "{"code": 200, "data": [...]}"
// @Router /api/address [get]
// @Security Bearer
func (e AddressController) GetPage(c *gin.Context) {
	s := service.Address{}
	req := dto.AddressGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	if req.AddressType != model.AddressTypeConsignee && req.AddressType != model.AddressTypeSender {
		err = errors.New("地址类型错误")
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	list := make([]model.Address, 0)
	var count int64

	err = s.GetPage(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// Get 通过id获取地址
// @Summary 通过id获取地址
// @Description 通过id获取地址
// @Tags 地址
// @Param id path string true "地址id"
// @Success 200 {object} response.Response{data=model.Address} "{"code": 200, "data": [...]}"
// @Router /api/address/{id} [get]
// @Security Bearer
func (e AddressController) Get(c *gin.Context) {
	s := service.Address{}
	req := dto.AddressGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.Address
	p := actions.GetPermissionFromContext(c)

	//数据权限检查
	err = s.Get(&req, &object, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加地址
// @Summary 添加地址
// @Description 添加地址
// @Tags 地址
// @Accept  application/json
// @Product application/json
// @Param data body dto.AddressInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/address [post]
// @Security Bearer
func (e AddressController) Insert(c *gin.Context) {
	s := service.Address{}
	req := dto.AddressInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)
	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	req.SetDeptId(p.DeptId)
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改地址
// @Summary 修改地址
// @Description 修改地址
// @Tags 地址
// @Accept  application/json
// @Product application/json
// @Param data body dto.AddressUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/address [put]
// @Security Bearer
func (e AddressController) Update(c *gin.Context) {
	s := service.Address{}
	req := dto.AddressUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	p := actions.GetPermissionFromContext(c)

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除地址
// @Summary 删除地址
// @Description 删除地址
// @Tags 地址
// @Accept  application/json
// @Product application/json
// @Param data body dto.AddressDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/address [delete]
// @Security Bearer
func (e AddressController) Delete(c *gin.Context) {
	s := service.Address{}
	req := dto.AddressDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
