package controller

import (
	"cold-logistics/app/admin/model"
	"cold-logistics/app/admin/service"
	"cold-logistics/app/admin/service/dto"
	"cold-logistics/common/actions"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"gogs.baozhida.cn/zoie/OAuth-core/api"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/jwtauth/user"
	_ "gogs.baozhida.cn/zoie/OAuth-core/pkg/response"
)

type CompanyController struct {
	api.Api
}

// GetPage 获取公司列表
// @Summary 获取公司列表
// @Description 获取公司列表（用户公司及子公司）
// @Tags 公司
// @Param name query string false "公司名称"
// @Success 200 {object} response.Response{data=response.Page{list=[]model.SysDept}} "{"code": 200, "data": [...]}"
// @Router /api/company [get]
// @Security Bearer
func (e CompanyController) GetPage(c *gin.Context) {
	s := service.Company{}
	req := dto.CompanyGetPageReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	p.DeptId = user.GetDeptId(c)
	list := make([]model.SysDept, 0)
	var count int64
	err = s.GetPage(&req, &list, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), req.GetPageIndex(), req.GetPageSize(), "查询成功")
}

// GetAll 添加用户-获取公司列表
// @Summary 添加用户-获取公司列表
// @Description 添加用户-获取公司列表
// @Tags 公司
// @Param id path string true "公司id"
// @Success 200 {object} response.Response{data=model.SysDept} "{"code": 200, "data": [...]}"
// @Router /api/company/{id} [get]
// @Security Bearer
func (e CompanyController) GetAll(c *gin.Context) {
	s := service.Company{}
	req := dto.CompanyGetAllReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.Query).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	//数据权限检查
	p := actions.GetPermissionFromContext(c)
	p.DeptId = user.GetDeptId(c)
	list := make([]model.SysDept, 0)
	var count int64
	err = s.GetAll(&req, &list, &count, p)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.PageOK(list, int(count), 0, 0, "查询成功")
}

// Get 通过id获取公司
// @Summary 通过id获取公司
// @Description 通过id获取公司
// @Tags 公司
// @Param id path string true "公司id"
// @Success 200 {object} response.Response{data=model.SysDept} "{"code": 200, "data": [...]}"
// @Router /api/company/{id} [get]
// @Security Bearer
func (e CompanyController) Get(c *gin.Context) {
	s := service.Company{}
	req := dto.CompanyGetReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, nil).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}
	var object model.SysDept
	//数据权限检查
	err = s.Get(&req, &object)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}

	e.OK(object, "查询成功")
}

// Insert 添加公司
// @Summary 添加公司
// @Description 添加公司
// @Tags 公司
// @Accept  application/json
// @Product application/json
// @Param data body dto.CompanyInsertReq true "data"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/company [post]
// @Security Bearer
func (e CompanyController) Insert(c *gin.Context) {
	s := service.Company{}
	req := dto.CompanyInsertReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	// 设置创建人
	req.SetCreateBy(user.GetUserId(c))
	err = s.Insert(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "创建成功")
}

// Update 修改公司
// @Summary 修改公司
// @Description 修改公司
// @Tags 公司
// @Accept  application/json
// @Product application/json
// @Param data body dto.CompanyUpdateReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "添加成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "添加失败"}"
// @Router /api/company [put]
// @Security Bearer
func (e CompanyController) Update(c *gin.Context) {
	s := service.Company{}
	req := dto.CompanyUpdateReq{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req).
		MakeService(&s.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	req.SetUpdateBy(user.GetUserId(c))
	err = s.Update(&req)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "更新成功")
}

// Delete 删除公司
// @Summary 删除公司
// @Description 删除公司
// @Tags 公司
// @Accept  application/json
// @Product application/json
// @Param data body dto.CompanyDeleteReq true "body"
// @Success 200 {string} string	"{"code": 200, "message": "删除成功"}"
// @Success 200 {string} string	"{"code": -1, "message": "删除失败"}"
// @Router /api/company [delete]
// @Security Bearer
func (e CompanyController) Delete(c *gin.Context) {
	s := service.Company{}
	req := dto.CompanyDeleteReq{}
	userSvc := service.SysUser{}
	err := e.MakeContext(c).
		MakeOrm().
		Bind(&req, binding.JSON, nil).
		MakeService(&s.Service).
		MakeService(&userSvc.Service).
		Errors
	if err != nil {
		e.Logger.Error(err)
		e.Error(500, err, err.Error())
		return
	}

	var count int64
	err = userSvc.GetCount(&dto.SysUserGetCountReq{DeptIds: []int{req.Id}}, &count)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	if count > 0 {
		e.Error(500, err, "有用户关联，禁止删除！")
		return
	}

	//数据权限检查
	//p := actions.GetPermissionFromContext(c)
	err = s.Remove(&req, nil)
	if err != nil {
		e.Error(500, err, err.Error())
		return
	}
	e.OK(req.GetId(), "删除成功")
}
