package service

import (
	"cold-logistics/app/admin/model"
	"cold-logistics/app/admin/service/dto"
	"cold-logistics/common/actions"
	cDto "cold-logistics/common/dto"
	"cold-logistics/common/global"
	"errors"
	"gogs.baozhida.cn/zoie/OAuth-core/service"
	"gorm.io/gorm"
)

type Car struct {
	service.Service
}

// GetPage 获取Car列表
func (e *Car) GetPage(c *dto.CarGetPageReq, list *[]model.Car, count *int64, p *actions.DataPermission) error {
	var err error
	var data model.Car

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			actions.Permission(data.TableName(), p),
		).
		Preload("User").
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}

// Get 获取Car对象
func (e *Car) Get(d *dto.CarGetReq, warehouseModel *model.Car, p *actions.DataPermission) error {
	err := e.Orm.
		Scopes(actions.Permission(warehouseModel.TableName(), p)).
		Preload("User").
		First(warehouseModel, d.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}

	return nil
}

// Insert 创建Car对象
func (e *Car) Insert(c *dto.CarInsertReq) error {
	var err error
	var data model.Car

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var i int64
	err = tx.Model(&data).Where("car_no = ?", c.CarNo).Count(&i).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	if i > 0 {
		err = errors.New("车牌号已存在！")
		e.Log.Errorf("db error: %s", err)
		return err
	}
	var j int64
	err = tx.Model(&data).Where("user_id = ?", c.UserId).Count(&j).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	if j > 0 {
		err = errors.New("该用户已绑定其他车辆！")
		e.Log.Errorf("db error: %s", err)
		return err
	}

	// 添加仓库
	c.Generate(&data)
	err = tx.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	c.Id = data.Id

	return nil

}

// Update 修改Car对象
func (e *Car) Update(c *dto.CarUpdateReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var warehouseModel = model.Car{}
	// 查询仓库是否存在
	err = e.Orm.Scopes(actions.Permission(warehouseModel.TableName(), p)).
		First(&warehouseModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	c.Generate(&warehouseModel)
	err = tx.Save(&warehouseModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = warehouseModel.Id
	return nil
}

// Remove 删除Car
func (e *Car) Remove(c *dto.CarDeleteReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var warehouseModel model.Car

	// 查询仓库是否存在
	err = e.Orm.Scopes(actions.Permission(warehouseModel.TableName(), p)).
		First(&warehouseModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.DeleteNotFoundOrNoPermissionErr
		}
		return global.DeleteFailedErr
	}

	db := tx.Delete(&warehouseModel)

	if err = db.Error; err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.DeleteFailedErr
	}
	if db.RowsAffected == 0 {
		return global.DeleteNotFoundOrNoPermissionErr
	}

	return nil
}
