package service

import (
	"cold-logistics/app/admin/model"
	"cold-logistics/app/admin/service/dto"
	"cold-logistics/common/actions"
	cDto "cold-logistics/common/dto"
	"cold-logistics/common/global"
	"errors"
	"gogs.baozhida.cn/zoie/OAuth-core/service"
	"gorm.io/gorm"
)

type Warehouse struct {
	service.Service
}

// GetPage 获取Warehouse列表
func (e *Warehouse) GetPage(c *dto.WarehouseGetPageReq, list *[]model.Warehouse, count *int64, p *actions.DataPermission) error {
	var err error
	var data model.Warehouse

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			actions.Permission(data.TableName(), p),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}

// Get 获取Warehouse对象
func (e *Warehouse) Get(d *dto.WarehouseGetReq, warehouseModel *model.Warehouse, p *actions.DataPermission) error {
	err := e.Orm.
		Scopes(actions.Permission(warehouseModel.TableName(), p)).
		First(warehouseModel, d.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}
	userWarehouse := make([]model.UserWarehouse, 0)
	err = e.Orm.Model(&model.UserWarehouse{}).Where("warehouse_id = ?", d.GetId()).Preload("User").Find(&userWarehouse).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	warehouseModel.UserList = userWarehouse

	return nil
}

// Insert 创建Warehouse对象
func (e *Warehouse) Insert(c *dto.WarehouseInsertReq) error {
	var err error
	var data model.Warehouse

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	// 添加仓库
	c.Generate(&data)
	err = tx.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	c.Id = data.Id

	// 添加用户和仓库的关联
	var userList []model.SysUser
	err = e.Orm.Where("id in (?) and dept_id = ?", c.UserIds, c.DeptId).Find(&userList).Error
	if err != nil {
		return global.GetFailedErr
	}
	if len(userList) == 0 {
		return errors.New("用户不存在！")
	}

	var userWarehouseList []model.UserWarehouse
	// 添加用户和仓库的关联
	for _, v := range userList {
		userWarehouseObj := model.UserWarehouse{
			UserId:      v.Id,
			WarehouseId: data.Id,
		}
		userWarehouseObj.SetCreateBy(c.CreateBy)
		userWarehouseList = append(userWarehouseList, userWarehouseObj)
	}
	err = tx.Save(&userWarehouseList).Error
	if err != nil {
		e.Log.Errorf("save userWarehouse userIds error, %s", err)
		return global.CreateFailedErr
	}
	return nil

}

// Update 修改Warehouse对象
func (e *Warehouse) Update(c *dto.WarehouseUpdateReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var warehouseModel = model.Warehouse{}
	// 查询仓库是否存在
	err = e.Orm.Scopes(actions.Permission(warehouseModel.TableName(), p)).
		First(&warehouseModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	c.Generate(&warehouseModel)
	err = tx.Save(&warehouseModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	c.Id = warehouseModel.Id

	// 删除用户和仓库的关联
	err = tx.Where("warehouse_id = ?", warehouseModel.Id).Delete(&model.UserWarehouse{}).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}

	// 添加用户和仓库的关联
	var userList []model.SysUser
	err = e.Orm.Where("id in (?) and dept_id = ?", c.UserIds, warehouseModel.DeptId).Find(&userList).Error
	if err != nil {
		return global.GetFailedErr
	}
	if len(userList) == 0 {
		return errors.New("用户不存在！")
	}

	var userWarehouseList []model.UserWarehouse
	// 添加用户和仓库的关联
	for _, v := range userList {
		userWarehouseObj := model.UserWarehouse{
			UserId:      v.Id,
			WarehouseId: warehouseModel.Id,
		}
		userWarehouseObj.SetCreateBy(c.UpdateBy)
		userWarehouseList = append(userWarehouseList, userWarehouseObj)
	}
	err = tx.Save(&userWarehouseList).Error
	if err != nil {
		e.Log.Errorf("save userWarehouse userIds error, %s", err)
		return global.CreateFailedErr
	}

	return nil
}

// Remove 删除Warehouse
func (e *Warehouse) Remove(c *dto.WarehouseDeleteReq, p *actions.DataPermission) error {
	var err error

	tx := e.Orm.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	var warehouseModel model.Warehouse

	// 查询仓库是否存在
	err = e.Orm.Scopes(actions.Permission(warehouseModel.TableName(), p)).
		First(&warehouseModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.DeleteNotFoundOrNoPermissionErr
		}
		return global.DeleteFailedErr
	}

	db := tx.Delete(&warehouseModel)

	if err = db.Error; err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.DeleteFailedErr
	}
	if db.RowsAffected == 0 {
		return global.DeleteNotFoundOrNoPermissionErr
	}

	return nil
}
